@@ -7,11 +7,9 @@
 #'   patterns for faster matching.
 #' @inheritParams pattern
 #' @param types unique types of tokens obtained by \code{\link{types}}
+#' @param keep_nomatch keep patterns not found
 #' @inheritParams valuetype
 #' @param case_insensitive if \code{TRUE}, ignores case when matching
-#' @param index If \code{NULL}, index is constructed automatically. It also
-#'   accept index constructed by \code{index_types}. In that case, \code{types},
-#'   \code{valuetype} and \code{case_insensitive} should be NULL.
 #' @return  \code{pattern2id} returns a list of integer vectors containing type
 #'   IDs
 #' @keywords internal
@@ -25,80 +23,62 @@
 #' pats_glob <- list(c("a*", "b*"), c("c"), c("d"))
 #' pattern2id(pats_glob, types, "glob", case_insensitive = TRUE)
 #' 
-pattern2id <- function(pattern, types = NULL, valuetype = NULL,
-                       case_insensitive = NULL, index = NULL) {
-    
+#' @export
+pattern2id <- function(pattern, types, valuetype = c("glob", "fixed", "regex"),
+                       case_insensitive = TRUE, keep_nomatch = FALSE) {
     
     if (!length(pattern)) return(list())
-    pattern <- lapply(pattern, stri_trans_nfc) # normalize unicode
     
-    if (is.null(index)) {
-        if (is.null(types)) stop("types cannot be NULL when index is not provided")
-        if (is.null(valuetype)) stop("valuetype cannot be NULL when index is not provided")
-        if (is.null(case_insensitive)) stop("case_insensitive cannot be NULL when index is not provided")
-        if (!is.character(types)) stop("types must be a character vector")
-        
-        # glob is treated as fixed if neither * or ? is found
-        pattern_unlist <- unlist(pattern, use.names = FALSE)
-        if (!valuetype %in% c("glob", "fixed", "regex"))
-            stop('valuetype should be "glob", "fixed" or "regex"')
-        if (valuetype == "glob" &&
-            !any(stri_detect_fixed(pattern_unlist, "*")) &&
-            !any(stri_detect_fixed(pattern_unlist, "?"))) {
-            valuetype <- "fixed"
-        }
-        max_len <- max(stri_length(unlist(pattern, use.names = FALSE)))
-        index <- index_types(types, valuetype, case_insensitive, max_len) # index types for quick search
+    pattern <- lapply(pattern, stri_trans_nfc) # normalize unicode
+    stopifnot(is.character(types))
+    valuetype <- match.arg(valuetype)
+    
+    # glob is treated as fixed if neither * or ? is found
+    if (valuetype == "glob" && !is_glob(pattern))
+        valuetype <- "fixed"
+    
+    len <- stri_length(unlist(pattern, use.names = FALSE))
+    if (length(len)) {
+        max_len <- max(len)    
     } else {
-        if (!is.null(types)) stop("types must be NULL when index is provided")
-        if (!is.null(valuetype)) stop("valuetype must be NULL when index is provided")
-        if (!is.null(case_insensitive)) stop("case_insensitive must be NULL when index is provided")
+        max_len <- 0
     }
-
-    # use options in the index
+        
+    # construct glob or fixed index for quick search
+    index <- index_types(types, valuetype, case_insensitive, max_len)
     types_search <- attr(index, "types_search")
-    valuetype <- attr(index, "valuetype")
-    case_insensitive <- attr(index, "case_insensitive")
 
     # lowercases for case-insensitive search
-    if (valuetype != "regex" && case_insensitive) {
+    if (valuetype != "regex" && case_insensitive)
         pattern <- lapply(pattern, stri_trans_tolower)
-    }
 
-    # separate multi and single-entry patterns
-    len <- lengths(pattern)
-    pats_multi <- pattern[len > 1]
-    pats_single <- pattern[len == 1]
-
-    # process multi-entry patterns
-    ids <- list()
-    for (pat_multi in pats_multi) {
-        if (valuetype == "regex") {
-            ids_multi <- search_regex(pat_multi, types_search, case_insensitive)
-        } else if (valuetype == "glob") {
-            ids_multi <- search_glob(pat_multi, types_search, index)
+    temp <- vector("list", length(pattern)) 
+    for (i in seq_along(pattern)) {
+        if (length(pattern[[i]]) > 1) {
+            if (valuetype == "regex") {
+                temp[[i]] <- search_regex_multi(pattern[[i]], types_search, case_insensitive)
+            } else if (valuetype == "glob") {
+                temp[[i]] <- search_glob_multi(pattern[[i]], types_search, index)
+            } else {
+                temp[[i]] <- search_fixed_multi(pattern[[i]], types_search, index)
+            }
         } else {
-            ids_multi <- search_fixed(pat_multi, types_search, index)
+            if (valuetype == "regex") {
+                temp[[i]] <- as.list(search_regex(pattern[[i]], types_search, case_insensitive))
+            } else if (valuetype == "glob") {
+                temp[[i]] <- as.list(search_glob(pattern[[i]], types_search, index))
+            } else {
+                temp[[i]] <- as.list(search_fixed(pattern[[i]], types_search, index))
+            }
         }
-        ids <- c(ids, expand(ids_multi))
     }
-
-    # process single-entry patterns
-    if (length(pats_single) > 0) {
-        pats_single <- unlist(pats_single, use.names = FALSE)
-        if (valuetype == "regex") {
-            ids_single <- unlist(search_regex(pats_single, types_search, case_insensitive), use.names = FALSE)
-        } else if (valuetype == "glob") {
-            ids_single <- unlist(search_glob(pats_single, types_search, index), use.names = FALSE)
-        } else {
-            ids_single <- unlist(search_fixed(pats_single, types_search, index), use.names = FALSE)
-        }
-        ids <- c(ids, ids_single)
-    }
-    return(unique(ids))
+    if (keep_nomatch) #{
+        temp <- lapply(temp, function(x) if (!length(x)) list(integer()) else x)
+    result <- unlist(temp, FALSE, FALSE)
+    names(result) <- rep(names(pattern), lengths(temp))
+    return(result)
 }
 
-
 #' @rdname pattern2id
 #' @description \code{pattern2fixed} converts regex and glob patterns to fixed patterns.
 #' @inherit pattern2id
@@ -110,15 +90,12 @@ pattern2id <- function(pattern, types = NULL, valuetype = NULL,
 #' pattern <- list(c("^a$", "^b"), c("c"), c("d"))
 #' types <- c("A", "AA", "B", "BB", "BBB", "C", "CC")
 #' pattern2fixed(pattern, types, "regex", case_insensitive = TRUE)
-#' index <- index_types(types, "regex", case_insensitive = TRUE)
-#' pattern2fixed(pattern, index = index)
-pattern2fixed <- function(pattern, types = NULL, valuetype = NULL,
-                          case_insensitive = NULL, index = NULL) {
-    id <- pattern2id(pattern, types, valuetype, case_insensitive, index)
-    if (!is.null(index))
-        types <- attr(index, "types")
-    fixed <- lapply(id, function(x) types[x])
-    return(fixed)
+pattern2fixed <- function(pattern, types, valuetype = c("glob", "fixed", "regex"),
+                          case_insensitive = TRUE, keep_nomatch = FALSE) {
+    
+    temp <- pattern2id(pattern, types, valuetype, case_insensitive, keep_nomatch)
+    result <- lapply(temp, function(x) types[x])
+    return(result)
 }
 
 
@@ -126,59 +103,78 @@ pattern2fixed <- function(pattern, types = NULL, valuetype = NULL,
 #' 
 #' This is an internal function for \code{pattern2id} that select types using
 #' keys in index when available.
-#' @param patterns a list of "glob", "fixed" or "regex" patterns
+#' @param pattern a "glob", "fixed" or "regex" pattern
 #' @param types_search lowercased types when \code{case_insensitive=TRUE}, but
 #'   not used in glob and fixed matching as types are in the index.
 #' @param case_insensitive ignore case when matching, if \code{TRUE}, but not
 #'   used in glob and fixed matching as types are lowercased in the index.
 #' @param index index object created by \code{index_types}
 #' @keywords internal
-search_glob <- function(patterns, types_search, index) {
-    lapply(patterns, function(pattern, types_search, index) {
-        if (pattern == "") {
-            return(integer()) # return nothing for empty pattern
-        } else if (pattern == "*") {
-            return(seq_along(types_search)) # return all types when glob is *
+search_glob <- function(pattern, types_search, index) {
+    if (length(pattern) == 0)  {
+        return(integer())
+    } else if (pattern == "") {
+        return(0L)
+    } else if (pattern == "*") {
+        return(seq_along(types_search)) # return all types when glob is *
+    } else {
+        pos <- search_index(pattern, index)
+        if (length((pos))) {
+            #cat("Index search", pattern, "\n")
+            return(pos)
+        } else if (!is_indexed(pattern)) {
+            #cat("Regex search", pattern, "\n")
+            return(which(stri_detect_regex(types_search, utils::glob2rx(pattern),
+                                           case_insensitive = FALSE)))
         } else {
-            pos <- search_index(pattern, index)
-            if (length((pos))) {
-                #cat("Index search", pattern, "\n")
-                return(pos)
-            } else if (!is_indexed(pattern)) {
-                #cat("Regex search", pattern, "\n")
-                return(which(stri_detect_regex(types_search, utils::glob2rx(pattern),
-                                               case_insensitive = FALSE)))
-            } else {
-                #cat("Not found\n")
-                return(integer())
-            }
+            #cat("Not found\n")
+            return(integer())
         }
-    }, types_search, index)
+    }
+}
+
+#' @rdname search_glob
+#' @param patterns a list of "glob", "fixed" or "regex" patterns
+#' @keywords internal
+search_glob_multi <- function(patterns, types_search, index) {
+    expand(lapply(patterns, search_glob, types_search, index))
 }
 
 #' @rdname search_glob
 #' @keywords internal
-search_regex <- function(patterns, types_search, case_insensitive) {
-    lapply(patterns, function(pattern, types_search, case_insensitive) {
-        if (pattern == "") {
-            return(integer())
-        } else {
-            return(which(stri_detect_regex(types_search, pattern,
-                                           case_insensitive = case_insensitive)))
-        }
-    }, types_search, case_insensitive)
+search_regex <- function(pattern, types_search, case_insensitive) {
+    if (length(pattern) == 0)  {
+        return(integer())
+    } else if (pattern == "") {
+        return(0L)
+    } else {
+        return(which(stri_detect_regex(types_search, pattern,
+                                       case_insensitive = case_insensitive)))
+    }
 }
 
 #' @rdname search_glob
 #' @keywords internal
-search_fixed <- function(patterns, types_search, index) {
-    lapply(patterns, function(pattern, types_search, index) {
-        if (pattern == "") {
-            return(integer())
-        } else {
-            return(search_index(pattern, index))
-        }
-    }, types_search, index)
+search_regex_multi <- function(patterns, types_search, case_insensitive) {
+    expand(lapply(patterns, search_regex, types_search, case_insensitive))
+}
+
+#' @rdname search_glob
+#' @keywords internal
+search_fixed <- function(pattern, types_search, index) {
+    if (length(pattern) == 0)  {
+        return(integer())
+    } else if (pattern == "") {
+        return(0L)
+    } else {
+        return(search_index(pattern, index))
+    }
+}
+
+#' @rdname search_glob
+#' @keywords internal
+search_fixed_multi <- function(patterns, types_search, index) {
+    expand(lapply(patterns, search_fixed, types_search, index))
 }
 
 #' Index types for fastest "glob" or "fixed" pattern matches
@@ -280,9 +276,12 @@ index_types <- function(types, valuetype, case_insensitive, max_len = NULL) {
 #' @param index an index object created by \code{index_types}
 #' @seealso \code{\link{index_types}}
 #' @keywords internal
-search_index <- function(pattern, index){
+ search_index <- function(pattern, index){
     # use fmatch instead of names for quick access
-    index[[fastmatch::fmatch(pattern, attr(index, "key"))]]
+    result <- index[[fastmatch::fmatch(pattern, attr(index, "key"))]]
+    if (is.null(result))
+        result <- integer()
+    return(result)
 }
 
 #' Simpler and faster version of expand.grid() in base package
@@ -314,7 +313,7 @@ expand <- function(elem){
 #' Internal function for \code{select_types} to check if a glob pattern is
 #' indexed by
 #' \code{index_types}.
-#' @param x a glob pattern to be tested
+#' @param pattern a glob pattern to be tested
 #' @keywords internal
 is_indexed <- function(pattern) {
     pattern <- stri_sub(pattern, 1, -2)
@@ -330,6 +329,14 @@ is_indexed <- function(pattern) {
     }
 }
 
+#' Check if patterns contains glob wildcard
+#' @param pattern a glob pattern to be tested
+#' @keywords internal
+is_glob <- function(pattern) {
+    pattern <- unlist(pattern, use.names = FALSE)
+    return(any(stri_detect_fixed(pattern, "*")) || any(stri_detect_fixed(pattern, "?")))
+}
+
 # internal-only aliases for backward compatibility
 # TODO: this should be removed with in a year (by April 2019).
 regex2id <- pattern2id
